"""Flow Converter - Nazar YAML formatini Maestro formatina cevirir ve tersini yapar."""
from pathlib import Path
from typing import Dict, List, Optional

import yaml


# Nazar -> Maestro donusum haritalari
# Nazar formati: {action: "tapOn", target: "Login"}
# Maestro formati: - tapOn: "Login" veya - tapOn: {id: "login_btn"}

NAZAR_TO_MAESTRO_ACTIONS = {
    "launchApp": "launchApp",
    "navigate": "launchApp",  # Nazar navigate -> Maestro launchApp ile acilir
    "goBack": "back",
    "scrollDown": "scroll",
    "scrollUp": "scroll",
    "tapOn": "tapOn",
    "longPress": "longPressOn",
    "doubleTap": "doubleTapOn",
    "inputText": "inputText",
    "clearText": "clearText",
    "selectOption": "tapOn",
    "assertVisible": "assertVisible",
    "assertNotVisible": "assertNotVisible",
    "assertText": "assertVisible",
    "assertEnabled": "assertVisible",
    "assertDisabled": "assertVisible",
    "waitForVisible": "waitForAnimationToEnd",
    "screenshot": "takeScreenshot",
    "wait": "waitForAnimationToEnd",
    "runFlow": "runFlow",
}

MAESTRO_TO_NAZAR_ACTIONS = {
    "launchApp": "launchApp",
    "back": "goBack",
    "scroll": "scrollDown",
    "tapOn": "tapOn",
    "longPressOn": "longPress",
    "doubleTapOn": "doubleTap",
    "inputText": "inputText",
    "clearText": "clearText",
    "assertVisible": "assertVisible",
    "assertNotVisible": "assertNotVisible",
    "waitForAnimationToEnd": "waitForVisible",
    "takeScreenshot": "screenshot",
    "runFlow": "runFlow",
    "swipe": "scrollDown",
    "inputRandomText": "inputText",
    "inputRandomNumber": "inputText",
    "inputRandomEmail": "inputText",
    "inputRandomPersonName": "inputText",
    "openLink": "navigate",
    "pressKey": "tapOn",
    "eraseText": "clearText",
    "hideKeyboard": "wait",
    "evalScript": "wait",
    "copyTextFrom": "assertVisible",
    "repeat": "wait",
}


def _convert_nazar_step_to_maestro(step: Dict) -> Optional[Dict]:
    """Tek bir Nazar adimini Maestro formatina cevir.

    Nazar formati:
        action: tapOn
        target: "Login"
        value: "test@mail.com"  (inputText icin)

    Maestro formati:
        - tapOn: "Login"
        - inputText: "test@mail.com"  (hedefi ayri satirda)
    """
    action = step.get("action", "")
    target = step.get("target", "")
    value = step.get("value", "")
    maestro_action = NAZAR_TO_MAESTRO_ACTIONS.get(action)

    if not maestro_action:
        return None

    # launchApp
    if action == "launchApp":
        app_id = target or step.get("appId", "")
        if app_id:
            return {"launchApp": {"appId": app_id}}
        return {"launchApp": True}

    # navigate -> Maestro'da launchApp veya openLink
    if action == "navigate":
        return {"launchApp": {"appId": target}}

    # goBack
    if action == "goBack":
        return {"back": True}

    # scroll
    if action == "scrollDown":
        return {"scroll": True}
    if action == "scrollUp":
        return {"scroll": {"direction": "UP"}}

    # tapOn
    if action == "tapOn":
        if _looks_like_id(target):
            return {"tapOn": {"id": target}}
        return {"tapOn": target}

    # longPress
    if action == "longPress":
        if _looks_like_id(target):
            return {"longPressOn": {"id": target}}
        return {"longPressOn": target}

    # doubleTap
    if action == "doubleTap":
        if _looks_like_id(target):
            return {"doubleTapOn": {"id": target}}
        return {"doubleTapOn": target}

    # inputText
    if action == "inputText":
        result = {"inputText": value or ""}
        if target:
            if _looks_like_id(target):
                return {"tapOn": {"id": target}, "_next": result}
            return {"tapOn": target, "_next": result}
        return result

    # clearText
    if action == "clearText":
        if target:
            if _looks_like_id(target):
                return {"tapOn": {"id": target}, "_next": {"eraseText": 100}}
            return {"tapOn": target, "_next": {"eraseText": 100}}
        return {"eraseText": 100}

    # selectOption -> tapOn ile
    if action == "selectOption":
        steps = []
        if target:
            steps.append({"tapOn": target})
        if value:
            steps.append({"tapOn": value})
        return steps[0] if len(steps) == 1 else {"_multi": steps}

    # assertVisible
    if action == "assertVisible":
        if _looks_like_id(target):
            return {"assertVisible": {"id": target}}
        return {"assertVisible": target}

    # assertNotVisible
    if action == "assertNotVisible":
        if _looks_like_id(target):
            return {"assertNotVisible": {"id": target}}
        return {"assertNotVisible": target}

    # assertText -> assertVisible + icerik kontrolu
    if action == "assertText":
        if value:
            return {"assertVisible": value}
        return {"assertVisible": target}

    # assertEnabled / assertDisabled -> assertVisible olarak
    if action in ("assertEnabled", "assertDisabled"):
        if _looks_like_id(target):
            return {"assertVisible": {"id": target}}
        return {"assertVisible": target}

    # waitForVisible
    if action == "waitForVisible":
        timeout = step.get("timeout", 5000)
        if _looks_like_id(target):
            return {"assertVisible": {"id": target, "timeout": timeout}}
        return {"assertVisible": {"text": target, "timeout": timeout}}

    # screenshot
    if action == "screenshot":
        name = target or "screenshot"
        return {"takeScreenshot": name}

    # wait
    if action == "wait":
        duration = step.get("duration", 1000)
        return {"waitForAnimationToEnd": {"timeout": duration}}

    # runFlow
    if action == "runFlow":
        flow = step.get("flow", target)
        return {"runFlow": flow}

    return None


def _convert_maestro_step_to_nazar(step: Dict) -> Optional[Dict]:
    """Tek bir Maestro adimini Nazar formatina cevir."""
    if not isinstance(step, dict):
        return None

    for maestro_action, content in step.items():
        nazar_action = MAESTRO_TO_NAZAR_ACTIONS.get(maestro_action)
        if not nazar_action:
            continue

        result = {"action": nazar_action}

        # launchApp
        if maestro_action == "launchApp":
            if isinstance(content, dict):
                result["target"] = content.get("appId", "")
            elif isinstance(content, str):
                result["target"] = content
            return result

        # back
        if maestro_action == "back":
            return result

        # scroll
        if maestro_action == "scroll":
            if isinstance(content, dict):
                direction = content.get("direction", "DOWN")
                if direction.upper() == "UP":
                    result["action"] = "scrollUp"
            return result

        # tapOn, longPressOn, doubleTapOn
        if maestro_action in ("tapOn", "longPressOn", "doubleTapOn"):
            if isinstance(content, dict):
                result["target"] = (
                    content.get("id")
                    or content.get("text")
                    or content.get("label", "")
                )
            elif isinstance(content, str):
                result["target"] = content
            return result

        # inputText
        if maestro_action == "inputText":
            if isinstance(content, str):
                result["value"] = content
            elif isinstance(content, dict):
                result["value"] = content.get("text", "")
                result["target"] = content.get("id", "")
            return result

        # clearText / eraseText
        if maestro_action in ("clearText", "eraseText"):
            if isinstance(content, dict):
                result["target"] = content.get("id", "")
            return result

        # assertVisible, assertNotVisible
        if maestro_action in ("assertVisible", "assertNotVisible"):
            if isinstance(content, dict):
                result["target"] = (
                    content.get("text")
                    or content.get("id")
                    or content.get("label", "")
                )
            elif isinstance(content, str):
                result["target"] = content
            return result

        # takeScreenshot
        if maestro_action == "takeScreenshot":
            if isinstance(content, str):
                result["target"] = content
            return result

        # waitForAnimationToEnd
        if maestro_action == "waitForAnimationToEnd":
            if isinstance(content, dict):
                result["duration"] = content.get("timeout", 1000)
            else:
                result["duration"] = 1000
            return result

        # runFlow
        if maestro_action == "runFlow":
            if isinstance(content, str):
                result["flow"] = content
            elif isinstance(content, dict):
                result["flow"] = content.get("file", "")
            return result

        # Bilinmeyen aksiyonlar icin genel donusum
        if isinstance(content, str):
            result["target"] = content
        elif isinstance(content, dict):
            result["target"] = (
                content.get("id")
                or content.get("text")
                or str(content)
            )
        return result

    return None


def _looks_like_id(target: str) -> bool:
    """Hedefin bir element ID'si mi yoksa gorunen metin mi oldugunu tahmin et."""
    if not target:
        return False
    # Bosluk iceriyorsa buyuk ihtimal gorunen metin
    if " " in target:
        return False
    # snake_case veya camelCase ise buyuk ihtimal ID
    if "_" in target:
        return True
    if target[0].islower() and any(c.isupper() for c in target[1:]):
        return True
    # Sadece kucuk harflerden olusuyorsa ID olabilir
    if target.isidentifier() and target.islower() and len(target) > 2:
        return True
    return False


def convert_nazar_to_maestro(nazar_yaml_path: str, project_path: str = None) -> str:
    """Nazar YAML dosyasini Maestro formatina cevir.

    Args:
        nazar_yaml_path: Nazar formatindaki YAML dosyasinin yolu.
        project_path: Proje kok dizini (opsiyonel). Verilirse YAML dosyasinin
                      bu dizin icinde oldugu dogrulanir.

    Returns:
        Maestro YAML icerigi (str).

    Raises:
        ValueError: YAML dosyasi proje dizini disindaysa veya gecersizse.
        FileNotFoundError: YAML dosyasi bulunamazsa.
    """
    path = Path(nazar_yaml_path).resolve()

    if not path.exists():
        raise FileNotFoundError(
            "YAML dosyasi bulunamadi: {}".format(path)
        )

    # Proje dizini verildiyse, YAML dosyasinin proje icinde oldugunu dogrula
    if project_path is not None:
        resolved_project = Path(project_path).resolve()
        try:
            path.relative_to(resolved_project)
        except ValueError:
            raise ValueError(
                "YAML dosyasi proje dizini disinda: {} (proje: {})".format(
                    path, resolved_project
                )
            )

    with open(path, "r", encoding="utf-8") as f:
        data = yaml.safe_load(f)

    if not data or "steps" not in data:
        raise ValueError(
            "Gecersiz Nazar YAML: 'steps' alani bulunamadi"
        )

    # Maestro meta bilgileri
    maestro_steps = []

    # appId varsa ilk adim olarak launchApp ekle
    app_id = data.get("appId")
    if app_id:
        maestro_steps.append({"launchApp": {"appId": app_id}})

    # Adimlari cevir
    for step in data.get("steps", []):
        if not isinstance(step, dict):
            continue

        converted = _convert_nazar_step_to_maestro(step)
        if converted is None:
            # Yorum olarak ekle
            maestro_steps.append(
                {"_comment": "Cevirilemeyen adim: {}".format(step)}
            )
            continue

        # Multi-step donusum (_next veya _multi)
        if "_next" in converted:
            next_step = converted.pop("_next")
            maestro_steps.append(converted)
            maestro_steps.append(next_step)
        elif "_multi" in converted:
            for sub in converted["_multi"]:
                maestro_steps.append(sub)
        else:
            maestro_steps.append(converted)

    # Yorumlari temizle, _comment satirlarini kaldir
    clean_steps = []
    for s in maestro_steps:
        if "_comment" in s:
            continue
        clean_steps.append(s)

    # Maestro YAML olustur
    output_lines = []

    # Nazar meta bilgilerini Maestro yorumu olarak ekle
    name = data.get("name", "")
    if name:
        output_lines.append("# {}".format(name))
    desc = data.get("description", "")
    if desc:
        output_lines.append("# {}".format(desc))
    if name or desc:
        output_lines.append("")

    # appId'yi ust seviyeye yaz
    if app_id:
        output_lines.append("appId: {}".format(app_id))
        output_lines.append("---")

    # Adimlari YAML olarak yaz
    yaml_content = yaml.dump(
        clean_steps,
        default_flow_style=False,
        allow_unicode=True,
        sort_keys=False,
    )

    output_lines.append(yaml_content)
    return "\n".join(output_lines)


def convert_maestro_to_nazar(maestro_yaml_path: str, project_path: str = None) -> str:
    """Maestro YAML dosyasini Nazar formatina cevir.

    Args:
        maestro_yaml_path: Maestro formatindaki YAML dosyasinin yolu.
        project_path: Proje kok dizini (opsiyonel). Verilirse YAML dosyasinin
                      bu dizin icinde oldugu dogrulanir.

    Returns:
        Nazar YAML icerigi (str).

    Raises:
        ValueError: YAML dosyasi proje dizini disindaysa.
        FileNotFoundError: YAML dosyasi bulunamazsa.
    """
    path = Path(maestro_yaml_path).resolve()

    if not path.exists():
        raise FileNotFoundError(
            "YAML dosyasi bulunamadi: {}".format(path)
        )

    # Proje dizini verildiyse, YAML dosyasinin proje icinde oldugunu dogrula
    if project_path is not None:
        resolved_project = Path(project_path).resolve()
        try:
            path.relative_to(resolved_project)
        except ValueError:
            raise ValueError(
                "YAML dosyasi proje dizini disinda: {} (proje: {})".format(
                    path, resolved_project
                )
            )

    content = path.read_text(encoding="utf-8")

    # Maestro dosyasi birden fazla YAML dokumani icerebilir (--- ayiricisi)
    documents = list(yaml.safe_load_all(content))

    # Meta bilgileri ilk dokumandan al
    meta = {}
    steps_data = []

    for doc in documents:
        if doc is None:
            continue
        if isinstance(doc, dict):
            # Meta bilgi dokumani (appId vs.)
            if "appId" in doc:
                meta["appId"] = doc["appId"]
            if "name" in doc:
                meta["name"] = doc["name"]
        elif isinstance(doc, list):
            # Adimlar
            steps_data = doc

    # Eger tek dokumanli ve liste ise
    if not steps_data and documents:
        first = documents[0]
        if isinstance(first, list):
            steps_data = first
        elif isinstance(first, dict) and "appId" not in first:
            # Tek adimlik dosya olabilir
            steps_data = [first]

    # Nazar adimlarina cevir
    nazar_steps = []
    for step in steps_data:
        converted = _convert_maestro_step_to_nazar(step)
        if converted:
            nazar_steps.append(converted)

    # Nazar YAML olustur
    nazar_data = {
        "apiVersion": "nazar/v1",
        "name": meta.get("name", path.stem.replace("-", " ").title() + " Testi"),
        "description": "Maestro'dan donusturulmus test",
        "platform": "all",
        "priority": "high",
    }

    if meta.get("appId"):
        nazar_data["appId"] = meta["appId"]

    nazar_data["steps"] = nazar_steps

    return yaml.dump(
        nazar_data,
        default_flow_style=False,
        allow_unicode=True,
        sort_keys=False,
    )



"""Responsive Analyzer - Statik layout analizi (runtime gerektirmez).

Faz 5: Media query, viewport, touch target, flexbox/grid, responsive image analizi.
"""
import os
import re
from pathlib import Path
from typing import Dict, List, Tuple

from nazar.runners.base import BaseRunner, IGNORE_DIRS

# Web/stil dosyalari icin uzantilar
WEB_EXTS = {".css", ".scss", ".less", ".html", ".htm", ".vue", ".svelte"}
# Kaynak + web birlesik
ALL_UI_EXTS = {".js", ".jsx", ".ts", ".tsx", ".vue", ".svelte", ".css", ".scss", ".less", ".html", ".htm"}

# Yaygin responsive breakpoint'ler (px)
COMMON_BREAKPOINTS = {320, 375, 480, 576, 640, 768, 800, 834, 900, 960, 1024, 1080, 1200, 1280, 1366, 1440, 1536, 1920}

# Minimum tap target boyutu (px) - WCAG 2.5.5 / Apple HIG / Material Design
MIN_TAP_TARGET = 44


class ResponsiveAnalyzer(BaseRunner):
    """Statik responsive ve layout analizi."""

    def __init__(self, project_path: str):
        super().__init__(project_path)
        self._web_file_cache: List[str] = []

    def run_check(self, subtype: str, test: dict) -> Tuple[bool, str]:
        checks = {
            "fixed_dimensions": self._fixed_dimensions,
            "scroll_issues": self._scroll_issues,
            "responsive_patterns": self._responsive_patterns,
            "media_query_analysis": self._media_query_analysis,
            "viewport_meta": self._viewport_meta,
            "touch_target_size": self._touch_target_size,
            "flexbox_grid_usage": self._flexbox_grid_usage,
            "responsive_images": self._responsive_images,
        }
        fn = checks.get(subtype)
        if fn:
            return fn(test)
        return True, "SKIP"

    # ---- Yardimci: Web/stil dosyalarini tara ----
    def _web_files(self) -> List[str]:
        """CSS, SCSS, HTML gibi web dosyalarini topla."""
        if self._web_file_cache:
            return self._web_file_cache
        result = []
        for root_dir, dirs, files in os.walk(self.root):
            dirs[:] = [d for d in dirs if d not in IGNORE_DIRS]
            for f in files:
                if Path(f).suffix.lower() in WEB_EXTS:
                    result.append(os.path.relpath(os.path.join(root_dir, f), self.root))
        self._web_file_cache = result
        return result

    def _all_ui_files(self) -> List[str]:
        """Kaynak + web dosyalarini birlesik dondur (UI analizi icin)."""
        seen = set()
        result = []
        for f in self.src_files()[:300]:
            if Path(f).suffix.lower() in ALL_UI_EXTS and f not in seen:
                seen.add(f)
                result.append(f)
        for f in self._web_files()[:200]:
            if f not in seen:
                seen.add(f)
                result.append(f)
        return result

    def _should_skip(self, f: str) -> bool:
        """Test ve nazar dosyalarini atla."""
        return "test" in f.lower() or "spec" in f.lower() or f.startswith("nazar/")

    # ---- Mevcut kontroller ----

    def _fixed_dimensions(self, t: dict) -> Tuple[bool, str]:
        """Sabit pixel boyutlari (responsive degil)."""
        findings = []
        for f in self.src_files()[:200]:
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue
            for m in re.finditer(r'width:\s*(\d{3,})', content):
                val = int(m.group(1))
                if val >= 300 and val not in (1024, 768):  # bilinen sabitler haric
                    line = content[:m.start()].count("\n") + 1
                    findings.append({"value": val, "file": f, "line": line})

        if not findings:
            return True, "Sabit boyut sorunu yok"
        first = findings[0]
        return False, "{} sabit genislik: width:{} ({}:{})".format(
            len(findings), first["value"], first["file"], first["line"])

    def _scroll_issues(self, t: dict) -> Tuple[bool, str]:
        """ScrollView icinde FlatList gibi sorunlar."""
        findings = []
        for f in self.src_files()[:200]:
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue
            if re.search(r'<ScrollView[^>]*>[\s\S]{0,5000}<(?:FlatList|SectionList)', content):
                line = content.find("ScrollView")
                line_num = content[:line].count("\n") + 1 if line >= 0 else 1
                findings.append({"file": f, "line": line_num, "issue": "ScrollView icinde FlatList"})

        if not findings:
            return True, "Scroll sorunu yok"
        first = findings[0]
        return False, "{} scroll sorunu: {} ({}:{})".format(
            len(findings), first["issue"], first["file"], first["line"])

    def _responsive_patterns(self, t: dict) -> Tuple[bool, str]:
        """Responsive pattern kontrolu."""
        findings = []
        for f in self.src_files()[:200]:
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue
            # Dimensions.get yerine useWindowDimensions
            if "Dimensions.get" in content and "useWindowDimensions" not in content:
                line = content.find("Dimensions.get")
                line_num = content[:line].count("\n") + 1 if line >= 0 else 1
                findings.append({"file": f, "line": line_num,
                                "issue": "Dimensions.get yerine useWindowDimensions oneriliyor"})
            # flexDirection row + flexWrap yok
            if re.search(r'flexDirection\s*:\s*["\']row["\']', content):
                if "flexWrap" not in content:
                    findings.append({"file": f, "line": 0,
                                    "issue": "flexDirection:row ama flexWrap yok"})

        if not findings:
            return True, "Responsive pattern'ler uygun"
        first = findings[0]
        return False, "{} responsive sorunu: {} ({}:{})".format(
            len(findings), first["issue"], first["file"], first["line"])

    # ---- Faz 5: Yeni kontroller ----

    def _media_query_analysis(self, t: dict) -> Tuple[bool, str]:
        """Media query analizi - breakpoint kullanimi ve kalitesi.

        Kontrol ettikleri:
        - Projede media query var mi
        - Breakpoint degerleri standart mi
        - min-width vs max-width yaklasimlari (mobile-first kontrol)
        - Yeterli breakpoint cesitliligi (mobil, tablet, desktop)
        """
        all_breakpoints = []
        files_with_mq = 0
        max_width_count = 0
        min_width_count = 0
        mq_files = []

        for f in self._all_ui_files():
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue

            # CSS/SCSS media query: @media (min-width: 768px)
            css_mqs = re.findall(
                r'@media\s*[^{]*\(\s*(?:min|max)-width\s*:\s*(\d+)\s*px\s*\)',
                content, re.IGNORECASE
            )
            # JS/TS media query: matchMedia, useMediaQuery
            js_mqs = re.findall(
                r'(?:matchMedia|useMediaQuery)\s*\(\s*["\'][^"\']*(?:min|max)-width\s*:\s*(\d+)\s*px',
                content, re.IGNORECASE
            )
            # Styled-components / CSS-in-JS: @media
            styled_mqs = re.findall(
                r'[`"\']@media\s*[^`"\']*\(\s*(?:min|max)-width\s*:\s*(\d+)\s*px',
                content, re.IGNORECASE
            )

            found_bps = [int(v) for v in css_mqs + js_mqs + styled_mqs]
            if found_bps:
                files_with_mq += 1
                mq_files.append(f)
                all_breakpoints.extend(found_bps)

            # min-width vs max-width sayimi
            min_width_count += len(re.findall(r'min-width\s*:', content, re.IGNORECASE))
            max_width_count += len(re.findall(r'max-width\s*:', content, re.IGNORECASE))

        # Hic UI dosyasi yoksa SKIP
        if not self._all_ui_files():
            return True, "Web/UI dosyasi bulunamadi, media query analizi atlanıyor"

        issues = []

        # Hic media query yoksa
        if files_with_mq == 0:
            return False, "Projede hic media query (responsive breakpoint) bulunamadi - farkli ekran boyutlarina uyum saglanamaz"

        # Breakpoint cesitliligi kontrolu
        unique_bps = set(all_breakpoints)
        has_mobile = any(bp <= 480 for bp in unique_bps)
        has_tablet = any(600 <= bp <= 1024 for bp in unique_bps)
        has_desktop = any(bp >= 1200 for bp in unique_bps)

        coverage = sum([has_mobile, has_tablet, has_desktop])
        if coverage < 2:
            missing = []
            if not has_mobile:
                missing.append("mobil (<=480px)")
            if not has_tablet:
                missing.append("tablet (600-1024px)")
            if not has_desktop:
                missing.append("desktop (>=1200px)")
            issues.append("Eksik breakpoint araligi: {}".format(", ".join(missing)))

        # Standart olmayan breakpoint'ler
        non_standard = [bp for bp in unique_bps if bp not in COMMON_BREAKPOINTS]
        if non_standard and len(non_standard) > len(unique_bps) * 0.5:
            issues.append("{} standart disi breakpoint degeri".format(len(non_standard)))

        # Mobile-first kontrol (min-width tercih edilmeli)
        total_mq = min_width_count + max_width_count
        if total_mq > 3 and max_width_count > min_width_count * 2:
            issues.append("max-width agirlikli ({} max vs {} min) - mobile-first yaklasim (min-width) onerilir".format(
                max_width_count, min_width_count))

        if not issues:
            return True, "{} dosyada {} benzersiz breakpoint ile responsive yapilandirma uygun".format(
                files_with_mq, len(unique_bps))

        return False, "Media query sorunlari: {}".format("; ".join(issues))

    def _viewport_meta(self, t: dict) -> Tuple[bool, str]:
        """Viewport meta tag kontrolu.

        Kontrol ettikleri:
        - HTML dosyalarinda viewport meta tag var mi
        - width=device-width ayarli mi
        - initial-scale=1.0 ayarli mi
        - maximum-scale=1 veya user-scalable=no (erisilebilirlik sorunu)
        """
        html_files = []
        for f in self._web_files():
            if self._should_skip(f):
                continue
            if f.lower().endswith((".html", ".htm")):
                html_files.append(f)

        # Ayrica src_files'tan JSX/TSX'te Head/Helmet icinde viewport arayalim
        jsx_viewport_found = False
        for f in self.src_files()[:200]:
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue
            # React Helmet veya Next.js Head'de viewport
            if re.search(r'<(?:Helmet|Head)[^>]*>[\s\S]{0,5000}viewport', content, re.IGNORECASE):
                jsx_viewport_found = True
                break
            # next.js metadata export
            if re.search(r'viewport\s*[:=]\s*["\']width=device-width', content, re.IGNORECASE):
                jsx_viewport_found = True
                break

        if not html_files and not jsx_viewport_found:
            # Framework tabanli proje olabilir (React Native, Flutter vs)
            # HTML olmayan projelerde bu kontrol uygulanmaz
            has_web_framework = False
            for f in self.src_files()[:50]:
                content = self.read(f)
                if content and re.search(r'import.*(?:react-dom|next|nuxt|angular|svelte|vue)', content, re.IGNORECASE):
                    has_web_framework = True
                    break

            if not has_web_framework:
                return True, "Web projesi degil, viewport kontrolu atlanıyor"

            if jsx_viewport_found:
                return True, "Viewport ayari framework uzerinden yapilmis"
            return False, "Web framework tespit edildi ama viewport meta tag bulunamadi"

        if jsx_viewport_found and not html_files:
            return True, "Viewport ayari framework (Helmet/Head) uzerinden tanimli"

        issues = []
        files_ok = 0
        files_missing = []
        a11y_issues = []

        for f in html_files:
            content = self.read(f)
            if not content:
                continue

            viewport_match = re.search(
                r'<meta\s+[^>]*name\s*=\s*["\']viewport["\'][^>]*content\s*=\s*["\']([^"\']+)["\']',
                content, re.IGNORECASE
            )
            if not viewport_match:
                # Farkli siralama: content once olabilir
                viewport_match = re.search(
                    r'<meta\s+[^>]*content\s*=\s*["\']([^"\']*device-width[^"\']*)["\'][^>]*name\s*=\s*["\']viewport["\']',
                    content, re.IGNORECASE
                )

            if not viewport_match:
                files_missing.append(f)
                continue

            vp_content = viewport_match.group(1)

            # width=device-width kontrolu
            if "width=device-width" not in vp_content:
                issues.append("{}: viewport'ta width=device-width eksik".format(f))

            # initial-scale kontrolu
            if "initial-scale" not in vp_content:
                issues.append("{}: initial-scale tanimli degil".format(f))

            # Erisilebilirlik: user-scalable=no veya maximum-scale=1
            if re.search(r'user-scalable\s*=\s*no', vp_content, re.IGNORECASE):
                a11y_issues.append("{}: user-scalable=no - kullanicilar yakinlastirma yapamaz (WCAG 1.4.4)".format(f))
            if re.search(r'maximum-scale\s*=\s*1(?:\.0)?(?:\s|,|$)', vp_content):
                a11y_issues.append("{}: maximum-scale=1 - zoom kisitli (WCAG 1.4.4)".format(f))

            if not issues and not a11y_issues:
                files_ok += 1

        all_issues = []
        if files_missing:
            all_issues.append("{} HTML dosyasinda viewport meta tag eksik: {}".format(
                len(files_missing), ", ".join(files_missing[:3])))
        all_issues.extend(issues[:3])
        all_issues.extend(a11y_issues[:2])

        if not all_issues:
            return True, "{} HTML dosyasinda viewport dogru yapilandirilmis".format(files_ok)

        return False, "Viewport sorunlari: {}".format("; ".join(all_issues))

    def _touch_target_size(self, t: dict) -> Tuple[bool, str]:
        """Dokunma hedefi boyut kontrolu (min 44x44px).

        WCAG 2.5.5 / Apple HIG / Material Design onerisine gore
        interaktif elementler en az 44x44 piksel olmali.
        """
        findings = []

        for f in self._all_ui_files():
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue

            # CSS'te kucuk buton/link boyutlari
            # Pattern: width/height/min-width/min-height deger < 44px (interaktif elementlerde)
            small_targets = re.finditer(
                r'(?:(?:button|a|input|\.btn|\.tab|\.icon-btn|\.nav-link|\.clickable|\.touchable)[^{]*\{[^}]{0,2000}'
                r'(?:width|height|min-width|min-height)\s*:\s*(\d{1,2})px)',
                content, re.IGNORECASE
            )
            for m in small_targets:
                val = int(m.group(1))
                if val < MIN_TAP_TARGET and val > 0:
                    line = content[:m.start()].count("\n") + 1
                    findings.append({"file": f, "line": line, "size": val})

            # React Native: style icinde kucuk width/height (Touchable, Pressable, Button)
            rn_patterns = re.finditer(
                r'<(?:TouchableOpacity|TouchableHighlight|Pressable|Button)[^>]*'
                r'style\s*=\s*\{[^}]{0,2000}(?:width|height)\s*:\s*(\d{1,2})',
                content
            )
            for m in rn_patterns:
                val = int(m.group(1))
                if val < MIN_TAP_TARGET and val > 0:
                    line = content[:m.start()].count("\n") + 1
                    findings.append({"file": f, "line": line, "size": val})

            # Inline style: style="width:30px;height:30px" on interactive elements
            inline_patterns = re.finditer(
                r'<(?:button|a|input)\s[^>]*style\s*=\s*["\'][^"\']*'
                r'(?:width|height)\s*:\s*(\d{1,2})px',
                content, re.IGNORECASE
            )
            for m in inline_patterns:
                val = int(m.group(1))
                if val < MIN_TAP_TARGET and val > 0:
                    line = content[:m.start()].count("\n") + 1
                    findings.append({"file": f, "line": line, "size": val})

            # Tailwind CSS: w-6, h-8 gibi kucuk boyutlar (interaktif elementlerde)
            # w-10 = 40px, w-11 = 44px, yani w-10 ve alti sorunlu
            tw_patterns = re.finditer(
                r'<(?:button|a|Link)\s[^>]*class(?:Name)?\s*=\s*["\'][^"\']*'
                r'(?:^|\s)(?:w|h)-(\d{1,2})(?:\s|["\'])',
                content, re.IGNORECASE
            )
            for m in tw_patterns:
                tw_val = int(m.group(1))
                px_val = tw_val * 4  # Tailwind: w-N = N*4px
                if px_val < MIN_TAP_TARGET and px_val > 0:
                    line = content[:m.start()].count("\n") + 1
                    findings.append({"file": f, "line": line, "size": px_val})

        if not findings:
            return True, "Dokunma hedefi boyutlari uygun (min {}px)".format(MIN_TAP_TARGET)

        first = findings[0]
        return False, "{} kucuk dokunma hedefi: {}px < {}px min ({}:{})".format(
            len(findings), first["size"], MIN_TAP_TARGET, first["file"], first["line"])

    def _flexbox_grid_usage(self, t: dict) -> Tuple[bool, str]:
        """Flexbox/Grid kullanim analizi.

        Kontrol ettikleri:
        - Projede flexbox veya CSS Grid kullaniliyor mu
        - Float-tabanli layout (eski yaklasim) var mi
        - Sabit pozisyonlama (absolute px) yerine esnek layout tercih edilmis mi
        """
        flex_count = 0
        grid_count = 0
        float_count = 0
        absolute_layout_count = 0
        files_analyzed = 0

        for f in self._all_ui_files():
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue
            files_analyzed += 1

            # Flexbox kullanimi
            flex_count += len(re.findall(r'display\s*:\s*flex', content, re.IGNORECASE))
            flex_count += len(re.findall(r'flexDirection\s*:', content))  # React Native
            flex_count += len(re.findall(r'(?:^|\s)(?:flex|inline-flex|items-center|justify-center|flex-row|flex-col)(?:\s|["\'])', content))  # Tailwind

            # CSS Grid kullanimi
            grid_count += len(re.findall(r'display\s*:\s*grid', content, re.IGNORECASE))
            grid_count += len(re.findall(r'grid-template-(?:columns|rows)\s*:', content, re.IGNORECASE))
            grid_count += len(re.findall(r'(?:^|\s)(?:grid-cols-|grid-rows-)', content))  # Tailwind

            # Float tabanli layout (eski yaklasim)
            float_count += len(re.findall(r'float\s*:\s*(?:left|right)', content, re.IGNORECASE))

            # Sabit absolute pozisyonlama (px ile)
            abs_matches = re.findall(
                r'position\s*:\s*absolute[^;]*;[^}]{0,2000}(?:left|top|right|bottom)\s*:\s*\d+px',
                content, re.IGNORECASE
            )
            absolute_layout_count += len(abs_matches)

        if files_analyzed == 0:
            return True, "UI dosyasi bulunamadi, layout analizi atlanıyor"

        issues = []
        responsive_score = flex_count + grid_count

        # Hic esnek layout yoksa
        if responsive_score == 0:
            issues.append("Projede flexbox veya CSS Grid kullanimi bulunamadi - responsive layout icin modern CSS oneriliyor")

        # Float agirlikli layout
        if float_count > 0 and float_count > responsive_score * 0.3:
            issues.append("{} float kullanimi tespit edildi - flexbox/grid'e gecis onerilir".format(float_count))

        # Cok fazla absolute pozisyonlama
        if absolute_layout_count > 5:
            issues.append("{} absolute pozisyonlama (px) - farkli ekran boyutlarinda bozulabilir".format(absolute_layout_count))

        if not issues:
            layout_info = []
            if flex_count:
                layout_info.append("flex:{}".format(flex_count))
            if grid_count:
                layout_info.append("grid:{}".format(grid_count))
            return True, "Esnek layout kullanimi uygun ({})".format(", ".join(layout_info) if layout_info else "tanimsiz")

        return False, "Layout sorunlari: {}".format("; ".join(issues))

    def _responsive_images(self, t: dict) -> Tuple[bool, str]:
        """Responsive gorsel (srcset, picture, responsive image) analizi.

        Kontrol ettikleri:
        - <img> etiketlerinde srcset kullaniliyor mu
        - <picture> elementi kullaniliyor mu
        - Sabit boyutlu gorseller var mi
        - Lazy loading kullaniliyor mu
        - next/image, gatsby-image gibi framework optimizasyonlari
        """
        img_total = 0
        img_with_srcset = 0
        picture_elements = 0
        framework_optimized = 0
        fixed_size_images = 0
        lazy_loading = 0
        files_with_images = []

        for f in self._all_ui_files():
            if self._should_skip(f):
                continue
            content = self.read(f)
            if not content:
                continue

            # <img> tag sayisi
            img_tags = re.findall(r'<img\s[^>]{0,2000}>', content, re.IGNORECASE)
            if not img_tags:
                # React: Image component (React Native)
                rn_images = re.findall(r'<Image\s[^>]*source\s*=', content)
                img_total += len(rn_images)
                if rn_images:
                    files_with_images.append(f)
                continue

            img_total += len(img_tags)
            files_with_images.append(f)

            for img in img_tags:
                # srcset kontrolu
                if re.search(r'srcset\s*=', img, re.IGNORECASE):
                    img_with_srcset += 1
                # sizes attribute
                # lazy loading
                if re.search(r'loading\s*=\s*["\']lazy["\']', img, re.IGNORECASE):
                    lazy_loading += 1
                # Sabit boyut (width="500" height="300" gibi buyuk px degerler)
                w_match = re.search(r'width\s*=\s*["\']?(\d+)', img, re.IGNORECASE)
                h_match = re.search(r'height\s*=\s*["\']?(\d+)', img, re.IGNORECASE)
                if w_match and int(w_match.group(1)) > 100:
                    if not re.search(r'srcset|sizes', img, re.IGNORECASE):
                        fixed_size_images += 1

            # <picture> elementi
            picture_elements += len(re.findall(r'<picture[\s>]', content, re.IGNORECASE))

            # Framework optimizasyonlari
            # next/image
            if re.search(r'import.*Image.*from\s+["\']next/image["\']', content):
                framework_optimized += 1
            # gatsby-image
            if re.search(r'import.*(?:GatsbyImage|StaticImage).*gatsby', content, re.IGNORECASE):
                framework_optimized += 1
            # nuxt-img
            if re.search(r'<(?:nuxt-img|NuxtImg)', content):
                framework_optimized += 1

        if img_total == 0:
            return True, "Gorsel elementi bulunamadi, responsive image kontrolu atlanıyor"

        issues = []

        # srcset kullanim orani
        non_optimized = img_total - img_with_srcset - framework_optimized - picture_elements
        if non_optimized > 0 and non_optimized > img_total * 0.5:
            issues.append("{}/{} gorsel srcset/picture olmadan kullaniliyor - farkli ekran yoğunluklari icin optimize degil".format(
                non_optimized, img_total))

        # Sabit boyutlu gorseller
        if fixed_size_images > 0:
            issues.append("{} gorsel sabit pixel boyutunda (srcset/sizes olmadan)".format(fixed_size_images))

        # Lazy loading eksikligi
        non_lazy = img_total - lazy_loading
        if non_lazy > 3 and lazy_loading == 0:
            issues.append("{} gorsel lazy loading kullanmiyor - sayfa yukleme performansini etkiler".format(non_lazy))

        if not issues:
            info = []
            if img_with_srcset:
                info.append("srcset:{}".format(img_with_srcset))
            if picture_elements:
                info.append("picture:{}".format(picture_elements))
            if framework_optimized:
                info.append("framework-optimized:{}".format(framework_optimized))
            if lazy_loading:
                info.append("lazy:{}".format(lazy_loading))
            return True, "{} gorsel responsive uyumlu ({})".format(
                img_total, ", ".join(info) if info else "uygun")

        return False, "Responsive gorsel sorunlari: {}".format("; ".join(issues))



"""Coverage Reporter - Test kapsami raporu olusturur."""
import json
import os
import re
from collections import Counter, defaultdict
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional


class CoverageReporter:
    """Tarama kapsami raporu.

    Kategori bazli gecme/kalma, dosya bazli sorun yogunlugu,
    guven dagilimi ve onceki taramayla trend verisi uretir.
    """

    # Guven esikleri
    CONFIDENCE_HIGH = 80
    CONFIDENCE_MEDIUM = 50

    def generate(
        self,
        results: List[Dict],
        plan: dict,
        output_path: Optional[str] = None,
    ) -> str:
        """Kapsam raporu olustur ve JSON olarak dondur.

        Args:
            results: Test sonuclari listesi.
            plan: Test plani dict'i.
            output_path: Opsiyonel dosya yolu. Verilirse JSON dosyaya yazilir.

        Returns:
            JSON string.
        """
        total = len(results)
        passed = sum(1 for r in results if r.get("passed"))
        failed = total - passed
        pass_rate = (passed / total * 100) if total > 0 else 0

        report = {
            "nazar_version": self._get_version(),
            "timestamp": datetime.now().isoformat(),
            "summary": {
                "total_checks": total,
                "passed": passed,
                "failed": failed,
                "pass_rate": round(pass_rate, 1),
                "grade": self._grade(pass_rate),
            },
            "category_coverage": self._category_coverage(results, plan),
            "file_issue_density": self._file_issue_density(results),
            "confidence_distribution": self._confidence_distribution(results),
            "severity_breakdown": self._severity_breakdown(results),
            "trend": self._trend_data(results, plan),
        }

        output = json.dumps(report, indent=2, ensure_ascii=False)
        if output_path:
            Path(output_path).parent.mkdir(parents=True, exist_ok=True)
            Path(output_path).write_text(output)
        return output

    # ------------------------------------------------------------------
    # Kategori kapsami
    # ------------------------------------------------------------------

    def _category_coverage(self, results: List[Dict], plan: dict) -> Dict:
        """Kategori bazinda gecme/kalma istatistikleri."""
        cats: Dict[str, Dict] = {}
        for r in results:
            cat = r.get("type", "other")
            if cat not in cats:
                cats[cat] = {"passed": 0, "failed": 0, "tests": []}
            if r.get("passed"):
                cats[cat]["passed"] += 1
            else:
                cats[cat]["failed"] += 1
            cats[cat]["tests"].append(r.get("name", ""))

        # Plandan beklenen kategorileri de ekle (0 sonuclu olanlar icin)
        for test in plan.get("tests", []):
            cat = test.get("type", "other")
            if cat not in cats:
                cats[cat] = {"passed": 0, "failed": 0, "tests": []}

        out = {}
        for cat, data in sorted(cats.items()):
            total = data["passed"] + data["failed"]
            rate = (data["passed"] / total * 100) if total > 0 else 0
            out[cat] = {
                "total": total,
                "passed": data["passed"],
                "failed": data["failed"],
                "pass_rate": round(rate, 1),
                "status": "clean" if rate == 100 else "partial" if rate >= 50 else "needs_attention",
            }
        return out

    # ------------------------------------------------------------------
    # Dosya bazli sorun yogunlugu
    # ------------------------------------------------------------------

    def _file_issue_density(self, results: List[Dict]) -> List[Dict]:
        """En cok sorun iceren dosyalari dondur."""
        file_issues: Dict[str, int] = Counter()
        file_severity: Dict[str, List[str]] = defaultdict(list)

        for r in results:
            if r.get("passed"):
                continue
            # detail icerisinden dosya yolunu cikarmaya calis
            detail = r.get("detail", "")
            target = r.get("target", "")
            file_path = target or self._extract_file_from_detail(detail)
            if file_path:
                file_issues[file_path] += 1
                file_severity[file_path].append(r.get("priority", "medium"))

        ranked = []
        for fp, count in file_issues.most_common(20):
            severities = file_severity.get(fp, [])
            ranked.append({
                "file": fp,
                "issue_count": count,
                "highest_severity": self._highest_severity(severities),
                "severities": dict(Counter(severities)),
            })
        return ranked

    # ------------------------------------------------------------------
    # Guven dagilimi
    # ------------------------------------------------------------------

    def _confidence_distribution(self, results: List[Dict]) -> Dict:
        """Bulgularin guven puani dagilimi (sadece failed icin)."""
        high = 0
        medium = 0
        low = 0
        scores = []

        for r in results:
            if r.get("passed"):
                continue
            conf = r.get("confidence", 50)
            if isinstance(conf, str):
                try:
                    conf = int(conf)
                except (ValueError, TypeError):
                    conf = 50
            scores.append(conf)
            if conf >= self.CONFIDENCE_HIGH:
                high += 1
            elif conf >= self.CONFIDENCE_MEDIUM:
                medium += 1
            else:
                low += 1

        total_findings = high + medium + low
        avg = round(sum(scores) / len(scores), 1) if scores else 0

        return {
            "high": {"count": high, "min_score": self.CONFIDENCE_HIGH, "label": "Kesin sorun"},
            "medium": {"count": medium, "min_score": self.CONFIDENCE_MEDIUM, "label": "Muhtemel sorun"},
            "low": {"count": low, "min_score": 0, "label": "Bilgilendirme"},
            "total_findings": total_findings,
            "average_confidence": avg,
        }

    # ------------------------------------------------------------------
    # Severity (oncelik) dagilimi
    # ------------------------------------------------------------------

    def _severity_breakdown(self, results: List[Dict]) -> Dict:
        """Oncelik bazinda toplam dagilim."""
        breakdown: Dict[str, Dict] = {}
        for level in ("critical", "high", "medium", "low"):
            breakdown[level] = {"total": 0, "passed": 0, "failed": 0}

        for r in results:
            pri = r.get("priority", "medium").lower()
            if pri not in breakdown:
                pri = "medium"
            breakdown[pri]["total"] += 1
            if r.get("passed"):
                breakdown[pri]["passed"] += 1
            else:
                breakdown[pri]["failed"] += 1

        return breakdown

    # ------------------------------------------------------------------
    # Trend verisi (onceki taramayla karsilastirma)
    # ------------------------------------------------------------------

    def _trend_data(self, results: List[Dict], plan: dict) -> Optional[Dict]:
        """Onceki tarama varsa karsilastirma verisi uret."""
        # Plan icerisinde proje yolu olabilir
        project_path = plan.get("project_path", "")
        if not project_path:
            # results'tan cikarmayi dene
            for r in results:
                detail = r.get("detail", "")
                target = r.get("target", "")
                if target and os.path.sep in target:
                    project_path = str(Path(target).parent)
                    break

        if not project_path:
            return None

        # .nazar/last-scan.json'u oku
        last_scan_path = Path(project_path) / ".nazar" / "last-scan.json"
        if not last_scan_path.exists():
            return {"available": False, "message": "Onceki tarama bulunamadi"}

        try:
            prev_data = json.loads(last_scan_path.read_text())
        except (json.JSONDecodeError, OSError):
            return {"available": False, "message": "Onceki tarama okunamadi"}

        prev_rate = prev_data.get("pass_rate", 0)
        prev_grade = prev_data.get("grade", "?")
        prev_total = prev_data.get("total", 0)
        prev_failed = prev_data.get("failed", 0)
        prev_timestamp = prev_data.get("timestamp", "")

        current_total = len(results)
        current_passed = sum(1 for r in results if r.get("passed"))
        current_rate = (current_passed / current_total * 100) if current_total > 0 else 0
        current_failed = current_total - current_passed

        delta = round(current_rate - prev_rate, 1)

        # Onceki failed isimleri
        prev_failed_names = set()
        for t in prev_data.get("tests", []):
            if not t.get("passed", True):
                prev_failed_names.add(t.get("name", ""))

        current_failed_names = set()
        for r in results:
            if not r.get("passed"):
                current_failed_names.add(r.get("name", ""))

        fixed = prev_failed_names - current_failed_names
        new_issues = current_failed_names - prev_failed_names

        return {
            "available": True,
            "previous": {
                "grade": prev_grade,
                "pass_rate": prev_rate,
                "total": prev_total,
                "failed": prev_failed,
                "timestamp": prev_timestamp,
            },
            "current": {
                "grade": self._grade(current_rate),
                "pass_rate": round(current_rate, 1),
                "total": current_total,
                "failed": current_failed,
            },
            "delta_rate": delta,
            "direction": "improving" if delta > 0 else "regressing" if delta < 0 else "stable",
            "fixed_count": len(fixed),
            "new_issues_count": len(new_issues),
            "fixed": sorted(fixed)[:10],
            "new_issues": sorted(new_issues)[:10],
        }

    # ------------------------------------------------------------------
    # Yardimci metodlar
    # ------------------------------------------------------------------

    @staticmethod
    def _extract_file_from_detail(detail: str) -> str:
        """Detail stringinden dosya yolunu cikar."""
        m = re.search(r'([a-zA-Z0-9_/\-\.()]+\.[a-zA-Z]{1,5})(?::(\d+))?', detail)
        if m:
            return m.group(1)
        return ""

    @staticmethod
    def _highest_severity(severities: List[str]) -> str:
        """Listeden en yuksek severity'yi dondur."""
        order = {"critical": 4, "high": 3, "medium": 2, "low": 1}
        if not severities:
            return "low"
        return max(severities, key=lambda s: order.get(s, 0))

    @staticmethod
    def _grade(rate: float) -> str:
        for min_r, g in [
            (97, "A+"), (93, "A"), (90, "A-"),
            (87, "B+"), (83, "B"), (80, "B-"),
            (77, "C+"), (73, "C"), (70, "C-"),
            (67, "D+"), (63, "D"), (60, "D-"),
            (0, "F"),
        ]:
            if rate >= min_r:
                return g
        return "F"

    @staticmethod
    def _get_version() -> str:
        try:
            from nazar import __version__
            return __version__
        except (ImportError, AttributeError):
            return "unknown"



"""Cross-File Analyzer - Dosyalar arasi iliskileri analiz eder."""
import re
import os
from pathlib import Path
from typing import List, Dict, Tuple, Set

from nazar.runners.base import BaseRunner


class CrossFileAnalyzer(BaseRunner):
    """Tek dosya analizinin yakalamedigi, dosyalar arasi sorunlari bulur."""

    def check_unused_exports(self, t: dict) -> Tuple[bool, str]:
        """Export edilen ama hicbir yerde import edilmeyen fonksiyon/component."""
        exports = {}  # {name: file}
        imports = set()

        for f in self.src_files()[:200]:
            content = self.read(f)
            # Export'lari topla
            for m in re.finditer(r"export\s+(?:default\s+)?(?:function|class|const|let)\s+(\w+)", content):
                exports[m.group(1)] = f
            for m in re.finditer(r"export\s*\{([^}]+)\}", content):
                for name in m.group(1).split(","):
                    name = name.strip().split(" as ")[0].strip()
                    if name:
                        exports[name] = f
            # Import'lari topla
            for m in re.finditer(r"import\s+(?:\{[^}]*\}|\w+)\s+from", content):
                for name in re.findall(r"\b(\w+)\b", m.group()):
                    if name not in ("import", "from"):
                        imports.add(name)

        unused = {name: f for name, f in exports.items()
                  if name not in imports and not name.startswith("_") and name not in ("default", "App", "app")}

        if len(unused) > 10:
            first = list(unused.items())[0]
            return False, f"{len(unused)} export edilip kullanilmayan: {first[0]} in {first[1]}"
        return True, f"{len(unused)} kullanilmayan export" if unused else "Temiz"

    def check_duplicate_secrets(self, t: dict) -> Tuple[bool, str]:
        """Ayni secret/config degeri birden fazla dosyada tekrar ediyor."""
        values = {}  # {value_hash: [files]}
        pattern = r"""(?:api[_-]?key|secret|token|password)\s*[:=]\s*['"]([^'"]{10,})['"]"""

        for f in self.src_files()[:100]:
            if "test" in f.lower() or ".env" in f:
                continue
            content = self.read(f)
            for m in re.finditer(pattern, content, re.IGNORECASE):
                val = m.group(1)
                if val not in ("process.env", "os.environ"):
                    values.setdefault(val[:20], []).append(f)

        duplicates = {v: files for v, files in values.items() if len(set(files)) > 1}
        if duplicates:
            first_key = list(duplicates.keys())[0]
            first_files = duplicates[first_key]
            return False, f"{len(duplicates)} secret birden fazla dosyada: {', '.join(set(first_files)[:3])}"
        return True, "Temiz"

    def check_orphan_components(self, t: dict) -> Tuple[bool, str]:
        """Tanimlanmis ama hicbir yerde kullanilmayan component dosyalari."""
        component_files = []
        import_targets = set()

        for f in self.src_files()[:200]:
            # Component dosyalarini bul
            if any(x in f.lower() for x in ["component", "screen", "page", "view", "modal"]):
                component_files.append(f)

            # Import hedeflerini topla
            content = self.read(f)
            for m in re.finditer(r"""from\s+['"]([^'"]+)['"]""", content):
                import_targets.add(m.group(1))

        orphans = []
        for cf in component_files:
            basename = Path(cf).stem
            is_imported = any(basename in target for target in import_targets)
            if not is_imported and basename not in ("index", "App", "_layout", "layout"):
                orphans.append(cf)

        if len(orphans) > 3:
            return False, f"{len(orphans)} kullanilmayan component dosyasi: {orphans[0]}"
        return True, "Temiz"

    def check_env_var_mismatch(self, t: dict) -> Tuple[bool, str]:
        """Kodda kullanilan ama .env.example'da olmayan env variable'lar."""
        used_vars = set()
        for f in self.src_files()[:100]:
            content = self.read(f)
            used_vars.update(re.findall(r"process\.env\.(\w+)", content))
            used_vars.update(re.findall(r"os\.environ(?:\.get)?\(?['\"](\w+)", content))

        if not used_vars:
            return True, "Env var kullanilmiyor"

        documented_vars = set()
        for env_file in [".env.example", ".env.sample", ".env.template"]:
            content = self.read(env_file)
            if content:
                for line in content.splitlines():
                    if "=" in line and not line.strip().startswith("#"):
                        documented_vars.add(line.split("=")[0].strip())

        # Filter out common/obvious ones
        undocumented = used_vars - documented_vars - {"NODE_ENV", "HOME", "PATH", "USER"}
        if len(undocumented) > 3 and documented_vars:
            return False, f"{len(undocumented)} env var dokumante edilmemis: {', '.join(list(undocumented)[:5])}"
        return True, "Temiz"

    def check_circular_imports(self, t: dict) -> Tuple[bool, str]:
        """Dosyalar arasi dairesel import tespiti (derin)."""
        graph = {}
        for f in self.src_files()[:200]:
            content = self.read(f)
            imports = set()
            for m in re.finditer(r"""(?:import|from)\s+['"](\.[^'"]+)['"]""", content):
                imports.add(m.group(1))
            graph[f] = imports

        # DFS ile cycle bul
        visited = set()
        rec_stack = set()
        cycles = []

        def dfs(node, path):
            visited.add(node)
            rec_stack.add(node)
            for imp in graph.get(node, []):
                # Resolve relative import
                resolved = None
                for candidate in graph:
                    if candidate.endswith(imp.lstrip("./") + ".ts") or candidate.endswith(imp.lstrip("./") + ".tsx") or candidate.endswith(imp.lstrip("./") + ".js"):
                        resolved = candidate
                        break
                if resolved:
                    if resolved in rec_stack:
                        cycles.append(f"{node} -> {resolved}")
                    elif resolved not in visited:
                        dfs(resolved, path + [node])
            rec_stack.discard(node)

        for node in graph:
            if node not in visited:
                dfs(node, [])

        if cycles:
            return False, f"{len(cycles)} dairesel import: {cycles[0]}"
        return True, "Temiz"

    def check_inconsistent_naming(self, t: dict) -> Tuple[bool, str]:
        """Dosya isimlendirme tutarsizligi (camelCase vs kebab-case vs snake_case)."""
        styles = {"camel": 0, "kebab": 0, "snake": 0, "pascal": 0}
        for f in self.src_files()[:200]:
            name = Path(f).stem
            if name.startswith("_") or name in ("index", "app", "main"):
                continue
            if "-" in name:
                styles["kebab"] += 1
            elif "_" in name:
                styles["snake"] += 1
            elif name[0].isupper():
                styles["pascal"] += 1
            elif any(c.isupper() for c in name[1:]):
                styles["camel"] += 1

        used_styles = {k: v for k, v in styles.items() if v > 0}
        if len(used_styles) > 2:
            detail = ", ".join(f"{k}:{v}" for k, v in sorted(used_styles.items(), key=lambda x: -x[1]))
            return False, f"Karisik dosya isimlendirme: {detail}"
        return True, "Tutarli"

    def check_api_auth_coverage(self, t: dict) -> Tuple[bool, str]:
        """API endpoint'lerinin auth middleware ile korunup korunmadigini kontrol et."""
        # Route tanimlari ve auth middleware'leri karsilastir
        routes_with_auth = 0
        routes_without_auth = 0

        for f in self.src_files()[:100]:
            content = self.read(f)
            lines = content.split("\n")
            for i, line in enumerate(lines):
                # Supabase edge function
                if "Deno.serve" in line or "serve(async" in line:
                    # Auth kontrolu var mi
                    fn_content = "\n".join(lines[max(0, i):min(len(lines), i+50)])
                    if "authorization" in fn_content.lower() or "getAuthUser" in fn_content or "auth" in fn_content.lower():
                        routes_with_auth += 1
                    else:
                        routes_without_auth += 1
                # Express/Fastify routes
                if re.search(r"(?:app|router)\.\s*(?:post|put|delete|patch)\s*\(", line):
                    context = "\n".join(lines[max(0, i-3):i+1])
                    if re.search(r"(?:auth|protect|guard|middleware|verify)", context, re.IGNORECASE):
                        routes_with_auth += 1
                    else:
                        routes_without_auth += 1

        if routes_without_auth > 0:
            total = routes_with_auth + routes_without_auth
            return False, f"{routes_without_auth}/{total} mutating route auth middleware eksik"
        return True, "Temiz"

    def get_all_checks(self) -> Dict[str, callable]:
        return {
            "unused_exports": self.check_unused_exports,
            "duplicate_secrets": self.check_duplicate_secrets,
            "orphan_components": self.check_orphan_components,
            "env_var_mismatch": self.check_env_var_mismatch,
            "circular_imports_deep": self.check_circular_imports,
            "inconsistent_naming": self.check_inconsistent_naming,
            "api_auth_coverage": self.check_api_auth_coverage,
        }

    def run_check(self, subtype: str, test: dict) -> Tuple[bool, str]:
        checks = self.get_all_checks()
        fn = checks.get(subtype)
        return fn(test) if fn else (True, "SKIP")



"""SARIF Reporter - GitHub Code Scanning entegrasyonu icin SARIF v2.1.0."""
import json
from datetime import datetime
from pathlib import Path
from typing import List, Dict


class SARIFReporter:
    """SARIF v2.1.0 cikti reporter'i."""

    SARIF_VERSION = "2.1.0"
    SARIF_SCHEMA = "https://docs.oasis-open.org/sarif/sarif/v2.1.0/cos02/schemas/sarif-schema-2.1.0.json"

    LEVEL_MAP = {
        "critical": "error",
        "high": "error",
        "medium": "warning",
        "low": "note",
    }

    def generate(self, results: List[Dict], plan: dict, output_path: str = None) -> str:
        """SARIF rapor olustur."""
        rules = []
        rule_ids = set()
        sarif_results = []

        for r in results:
            if r["passed"]:
                continue

            rule_id = f"nazar/{r.get('type', 'unknown')}/{r.get('subtype', 'check')}"
            if rule_id not in rule_ids:
                rule_ids.add(rule_id)
                rules.append({
                    "id": rule_id,
                    "name": r.get("subtype", "check"),
                    "shortDescription": {"text": r["name"]},
                    "defaultConfiguration": {
                        "level": self.LEVEL_MAP.get(r.get("priority", "medium"), "warning")
                    },
                })

            detail = r.get("detail", "")
            file_path = ""
            line = 1
            if ":" in detail:
                parts = detail.split(":")
                for p in parts:
                    if "/" in p or "." in p:
                        file_path = p.strip()
                    elif p.strip().isdigit():
                        line = int(p.strip())

            result = {
                "ruleId": rule_id,
                "level": self.LEVEL_MAP.get(r.get("priority", "medium"), "warning"),
                "message": {"text": f"{r['name']}: {detail}"},
            }

            if file_path:
                result["locations"] = [{
                    "physicalLocation": {
                        "artifactLocation": {"uri": file_path},
                        "region": {"startLine": line},
                    }
                }]

            sarif_results.append(result)

        sarif = {
            "$schema": self.SARIF_SCHEMA,
            "version": self.SARIF_VERSION,
            "runs": [{
                "tool": {
                    "driver": {
                        "name": "Nazar",
                        "version": "2.0.0",
                        "informationUri": "https://github.com/user/nazar",
                        "rules": rules,
                    }
                },
                "results": sarif_results,
                "invocations": [{
                    "executionSuccessful": True,
                    "endTimeUtc": datetime.utcnow().isoformat() + "Z",
                }],
            }],
        }

        output = json.dumps(sarif, indent=2, ensure_ascii=False)
        if output_path:
            Path(output_path).write_text(output)
        return output



"""UI Element Extractor - Projeden tum UI elementlerini tek seferde cikarir.

Shared Analysis Layer: Bu veri yazim, renk, font, a11y, i18n katmanlarina dagitilir.
"""
import os
import re
from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional

from nazar.runners.base import BaseRunner, IGNORE_DIRS


@dataclass
class UIElement:
    file: str
    line: int
    element_type: str       # Button, Text, TextInput, Image, View, TouchableOpacity...
    text: str = ""          # gorunen metin
    style: Dict = field(default_factory=dict)
    accessibility_label: str = ""
    test_id: str = ""
    props: Dict = field(default_factory=dict)
    placeholder: str = ""
    source: str = ""        # hangi framework (react, flutter, swift, kotlin)


class UIElementExtractor(BaseRunner):
    """Projeden tum UI elementlerini cikarir."""

    # JSX/TSX element pattern'leri
    JSX_ELEMENTS = {
        "Text": r'<Text[^>]*>([^<]*)</Text>',
        "Button": r'<(?:Button|TouchableOpacity|Pressable)[^>]*(?:title=["\']([^"\']+)["\']|>([^<]*))',
        "TextInput": r'<TextInput[^>]*',
        "Image": r'<Image[^>]*',
        "View": r'<(?:View|ScrollView|FlatList|SafeAreaView)[^>]*',
    }

    # Prop extraction
    PROP_PATTERNS = {
        "text": r'(?:title|label|children)=["\']([^"\']+)["\']',
        "placeholder": r'placeholder=["\']([^"\']+)["\']',
        "testID": r'testID=["\']([^"\']+)["\']',
        "accessibilityLabel": r'accessibilityLabel=["\']([^"\']+)["\']',
        "style_color": r'color:\s*["\']?(#[0-9A-Fa-f]{3,8}|rgb\([^)]+\))["\']?',
        "style_bg": r'backgroundColor:\s*["\']?(#[0-9A-Fa-f]{3,8}|rgb\([^)]+\))["\']?',
        "style_fontSize": r'fontSize:\s*(\d+)',
        "style_fontFamily": r'fontFamily:\s*["\']([^"\']+)["\']',
        "style_width": r'width:\s*(\d+)',
        "style_height": r'height:\s*(\d+)',
        "required": r'\brequired\b',
        "maxLength": r'maxLength=\{?(\d+)',
        "keyboardType": r'keyboardType=["\']([^"\']+)["\']',
    }

    def extract(self, source_files: List[str] = None) -> List[UIElement]:
        """Tum UI elementlerini cikar."""
        if source_files is None:
            source_files = self._ui_files()
        elements = []
        for f in source_files:
            content = self.read(f)
            if not content:
                continue
            ext = Path(f).suffix.lower()
            if ext in (".tsx", ".jsx", ".ts", ".js"):
                elements.extend(self._parse_jsx(f, content))
            elif ext == ".dart":
                elements.extend(self._parse_dart(f, content))
            elif ext == ".swift":
                elements.extend(self._parse_swift(f, content))
            elif ext == ".kt" or ext == ".java":
                elements.extend(self._parse_kotlin(f, content))
        return elements

    def _ui_files(self) -> List[str]:
        """UI dosyalarini bul (screen, component, page)."""
        ui_exts = {".tsx", ".jsx", ".ts", ".js", ".dart", ".swift", ".kt", ".java"}
        files = []
        for f in self.src_files():
            if Path(f).suffix.lower() in ui_exts:
                if "test" not in f.lower() and "spec" not in f.lower() and not f.startswith("nazar/"):
                    files.append(f)
        return files

    def _parse_jsx(self, file_path: str, content: str) -> List[UIElement]:
        """JSX/TSX dosyasini parse et."""
        elements = []
        lines = content.split("\n")

        for i, line in enumerate(lines, 1):
            # <Text>icerik</Text>
            for m in re.finditer(r'<Text[^>]*>([^<]+)</Text>', line):
                text = m.group(1).strip()
                if text and not text.startswith("{"):
                    el = UIElement(file=file_path, line=i, element_type="Text", text=text, source="react")
                    self._extract_props(el, line)
                    elements.append(el)

            # <Text>{t('key')}</Text> - i18n
            for m in re.finditer(r'<Text[^>]*>\{t\(["\']([^"\']+)["\']\)\}</Text>', line):
                el = UIElement(file=file_path, line=i, element_type="Text", text=m.group(1), source="react")
                el.props["i18n"] = True
                elements.append(el)

            # <Button title="..." /> veya <TouchableOpacity>
            for m in re.finditer(r'<(?:Button|TouchableOpacity|Pressable)[^>]*title=["\']([^"\']+)["\']', line):
                el = UIElement(file=file_path, line=i, element_type="Button", text=m.group(1), source="react")
                self._extract_props(el, line)
                elements.append(el)

            # <TextInput placeholder="..." />
            for m in re.finditer(r'<TextInput[^>]*placeholder=["\']([^"\']+)["\']', line):
                el = UIElement(file=file_path, line=i, element_type="TextInput", source="react")
                el.placeholder = m.group(1)
                el.text = m.group(1)
                self._extract_props(el, line)
                elements.append(el)

            # <Image />
            if "<Image" in line and "source" in line:
                el = UIElement(file=file_path, line=i, element_type="Image", source="react")
                self._extract_props(el, line)
                elements.append(el)

        return elements

    def _parse_dart(self, file_path: str, content: str) -> List[UIElement]:
        """Flutter/Dart dosyasini parse et."""
        elements = []
        lines = content.split("\n")
        for i, line in enumerate(lines, 1):
            # Text('icerik')
            for m in re.finditer(r"Text\(\s*['\"]([^'\"]+)['\"]", line):
                elements.append(UIElement(file=file_path, line=i, element_type="Text", text=m.group(1), source="flutter"))
            # ElevatedButton / TextButton
            for m in re.finditer(r"(?:ElevatedButton|TextButton|OutlinedButton)\(", line):
                el = UIElement(file=file_path, line=i, element_type="Button", source="flutter")
                # child: Text('...') bir sonraki satirda olabilir
                context = "\n".join(lines[max(0, i-1):min(len(lines), i+3)])
                tm = re.search(r"Text\(\s*['\"]([^'\"]+)['\"]", context)
                if tm:
                    el.text = tm.group(1)
                elements.append(el)
            # TextField
            if "TextField(" in line or "TextFormField(" in line:
                el = UIElement(file=file_path, line=i, element_type="TextInput", source="flutter")
                hm = re.search(r"hintText:\s*['\"]([^'\"]+)['\"]", "\n".join(lines[max(0, i-1):min(len(lines), i+5)]))
                if hm:
                    el.placeholder = hm.group(1)
                    el.text = hm.group(1)
                elements.append(el)
        return elements

    def _parse_swift(self, file_path: str, content: str) -> List[UIElement]:
        """SwiftUI dosyasini parse et."""
        elements = []
        lines = content.split("\n")
        for i, line in enumerate(lines, 1):
            for m in re.finditer(r'Text\("([^"]+)"\)', line):
                elements.append(UIElement(file=file_path, line=i, element_type="Text", text=m.group(1), source="swift"))
            for m in re.finditer(r'Button\("([^"]+)"\)', line):
                elements.append(UIElement(file=file_path, line=i, element_type="Button", text=m.group(1), source="swift"))
            if "TextField(" in line:
                tm = re.search(r'TextField\("([^"]*)"', line)
                el = UIElement(file=file_path, line=i, element_type="TextInput", source="swift")
                if tm:
                    el.placeholder = tm.group(1)
                    el.text = tm.group(1)
                elements.append(el)
        return elements

    def _parse_kotlin(self, file_path: str, content: str) -> List[UIElement]:
        """Jetpack Compose / Android dosyasini parse et."""
        elements = []
        lines = content.split("\n")
        for i, line in enumerate(lines, 1):
            for m in re.finditer(r'Text\(\s*(?:text\s*=\s*)?["\']([^"\']+)["\']', line):
                elements.append(UIElement(file=file_path, line=i, element_type="Text", text=m.group(1), source="kotlin"))
            for m in re.finditer(r'Button\(', line):
                el = UIElement(file=file_path, line=i, element_type="Button", source="kotlin")
                context = "\n".join(lines[max(0, i-1):min(len(lines), i+3)])
                tm = re.search(r'Text\(\s*["\']([^"\']+)["\']', context)
                if tm:
                    el.text = tm.group(1)
                elements.append(el)
        return elements

    def _extract_props(self, el: UIElement, line: str):
        """Bir satirdaki prop'lari extract et."""
        for prop_name, pattern in self.PROP_PATTERNS.items():
            m = re.search(pattern, line)
            if m:
                if prop_name == "accessibilityLabel":
                    el.accessibility_label = m.group(1) if m.lastindex else ""
                elif prop_name == "testID":
                    el.test_id = m.group(1) if m.lastindex else ""
                elif prop_name == "placeholder":
                    el.placeholder = m.group(1) if m.lastindex else ""
                elif prop_name == "style_color":
                    el.style["color"] = m.group(1) if m.lastindex else ""
                elif prop_name == "style_bg":
                    el.style["backgroundColor"] = m.group(1) if m.lastindex else ""
                elif prop_name == "style_fontSize":
                    el.style["fontSize"] = m.group(1) if m.lastindex else ""
                elif prop_name == "style_fontFamily":
                    el.style["fontFamily"] = m.group(1) if m.lastindex else ""
                elif prop_name == "required":
                    el.props["required"] = True
                elif prop_name == "maxLength":
                    el.props["maxLength"] = int(m.group(1)) if m.lastindex else 0
                elif prop_name == "keyboardType":
                    el.props["keyboardType"] = m.group(1) if m.lastindex else ""
